function plot_QP(X, H, f, sol_val_qp)
    % Compute unconstrained solution
    xunc = -inv(H) * f;
    fprintf('Unconstrained LP solution using CasADi = [%f %f]\n', xunc(1), xunc(2));
    
    % Cost function value comparison
    Func = (xunc' * H * xunc / 2 + f' * xunc);
    Fquad = (sol_val_qp' * H * sol_val_qp / 2 + f' * sol_val_qp);
    fprintf('Func = %f\nFquad = %f\n', Func, Fquad);
    
    % Generate quadratic cost in form of an ellipse
    [xxquad, yyquad] = createEllipse(H, f, Fquad);
    [xxquad2, yyquad2] = createEllipse(H, f, Fquad * (-2));
    [xxquad3, yyquad3] = createEllipse(H, f, Fquad * (0.5));
    
    % Plot the results
    figure;
    hold on;
    
    h4 = plot(xxquad, yyquad, 'k');
    h5 = plot(xxquad2, yyquad2, 'LineStyle', '--');
    h6 = plot(xxquad3, yyquad3, 'LineStyle', '--');
    h2 = scatter(sol_val_qp(1), sol_val_qp(2), 50, 'filled', 'r');
    h3 = scatter(xunc(1), xunc(2), 25, 'filled', 'g');
    
    opt = {'Color', 'b', 'Alpha', 0.4};
    h1 = plot(X, opt{:});
    
    % Add title and legend
    title('Quadratic cost constrained optimization problem');
    legend([h1 h2 h3 h4], 'Constraints', 'Constrained optimum', 'Unconstrained optimum', 'Quadratic cost', 'Location', 'best');
    % xlim([-7 7]);
    % ylim([-7 7]);
end

