clear all; close all; clc;
import casadi.*

%% Jacobian and Hessian
import casadi.*
x = MX.sym('x');
y = MX.sym('y');
w = x + y;
z = w * y;
z = 4 * z; % z = 4 * (x + y) * y

f = Function('f', {[x; y]}, {z}, struct('live_variables', false));
% f = Function('f', {x, y}, {z}, struct('live_variables', false)); % Same

% Jacobian
J = jacobian(f);
disp(J, true);

H = jacobian(jacobian(f));
disp(H, true);

% Define values for x and y
x_val = 1;
y_val = 2;

J_val = full(J([x_val; y_val], []))
H_val = full(H([x_val; y_val], [], []))

%% Rootfinding
nz = 3; nx = 3;
z  = SX.sym('x', nz);
x  = SX.sym('x', nx);
g0 = sin(x + z);
g1 = cos(x - z);
g  = Function('g', {z, x}, {g0, g1});
G  = rootfinder('G', 'newton', g);
disp(G);
x_val = randi(10, nz, 1);
z_guess = randi(10, nz, 1);
z_val = G(z_guess, x_val)

%% Integrator
SX_x = SX.sym('x', 2, 1);
SX_u = SX.sym('u', 1, 1);

A = [0, 1; 0, 0];
B = [0; 1];

ode = struct('x', SX_x, 'p', SX_u, 'ode', A * SX_x + B * SX_u);

T        = 10;
T_star   = 5;
T_sample = 0.1;
tsamples = 100;
opts     = struct('grid', linspace(0, T_sample, tsamples + 1));
F        = integrator('F', 'cvodes', ode, opts);

figure; grid on; hold on;

xi = [2; 0];
traj_vals = [xi];
for i = 1:floor(T / T_sample)
    if (T_sample * i < T_star)
        trajectory = F('x0', traj_vals(:, end), 'p', -1);
    else
        trajectory = F('x0', traj_vals(:, end), 'p', 1);
    end
    traj_vals = [traj_vals, full(trajectory.xf)];
end
%get the trajectory samples
plot(traj_vals(1, :), traj_vals(2, :));

%% If else instruction
% Saturate function
x       = SX.sym('x');
f_true  = Function('f_true', {x}, {cos(x)});
f_false = Function('f_false', {x}, {sin(x)});
f_cond  = Function.if_else('f_cond', f_true, f_false);
disp(f_cond);
f_cond(0, pi)
f_cond(1, pi)

% Saturate symbolic variable
sym_var = SX.sym('x', 3, 1);
u_max = [1; 1; 1];
u_min = [-1; -1; -1];

sym_var_s = SX.sym('xs', 3, 1);
for i = 1:length(sym_var)
    sym_var_s(i) = if_else(if_else(sym_var(i) < u_max(i), sym_var(i), u_max(i)) > u_min(i), ...
                           if_else(sym_var(i) < u_max(i), sym_var(i), u_max(i)), ...
                           u_min(i));
end

