clear all; close all; clc;
import casadi.*

%% LP problem
% Define constraints set
V = [-6.5,-0.25; -2, 3.45; 3.5, 3.45; 2.85,-4.5; -6.5,-0.25]; % vertices
X = Polyhedron('V', V); % requires installation of mpt3 toolbox

opti_var = casadi.Opti();   % Create the opti object
x = opti_var.variable(2);   % Define symbolic variable
c = opti_var.parameter(2);  % Define parameter

% Define the constraints
constraints = X.A * x <= X.b;
% Define the cost function
cost_LP = -(c' * x);

% Set up options for the solver (assuming you're using IPOPT)
options.ipopt.print_level = 0;  % Set the print level for IPOPT
options.ipopt.sb = 'yes';       % Enable some IPOPT specific option (sb)
options.print_time = 0;         % Disable printing of the solver time

% Apply the constraints
opti_var.subject_to(constraints);

% Define the cost function to minimize
opti_var.minimize(cost_LP);

% Set the solver to 'ipopt' and apply the options
opti_var.solver('ipopt', options);

% Solve the optimization problem
c_val = [3; 4];
opti_var.set_value(c, c_val);
solution = opti_var.solve();

% Extract the solution value (assuming 'x' is the variable you defined)
sol_val_lp = solution.value(opti_var.x);

% Print the solution
fprintf('LP solution using CasADi = [%f %f]\n', sol_val_lp(1), sol_val_lp(2));

plot_LP(X, sol_val_lp, c_val);

%% QP problem
opti_var = casadi.Opti();  % Create an optimization solver
x = opti_var.variable(2, 1);  % Define a variable with dimensions 2x1

H = [0.2, 0.01; 0.09, 0.4]; 
H = H * H';  % Ensures H is positive definite
f = [0.1; 0.9];

% Define constraints and cost function
constraints = X.A * x <= X.b;  
cost_QP = x' * H * x / 2 + f' * x;

% Solver options
options.ipopt.print_level = 0;
options.ipopt.sb = 'yes';
options.print_time = 0;

% Set up the solver
opti_var.subject_to(constraints);
opti_var.minimize(cost_QP);
opti_var.solver('ipopt', options);

% Solve the problem and measure computation time
timer = tic;
solution = opti_var.solve();
time2 = toc(timer);

% Extract solution values
sol_val_qp = solution.value(opti_var.x);
fprintf('QP solution using CasADi = [%f %f]\n', sol_val_qp(1), sol_val_qp(2));

plot_QP(X, H, f, sol_val_qp);
