clear all; close all; clc;
import casadi.*

%% Hanging chain
N = 40;     % Number of points (chosen by us)
m = 40 / N; % Weight of each section
D = 70 * N; % Stifness coeficient
g = 9.81;   % gravitation acceleration
L = 1;      % length of the chain

% Set up optimization environment
opti_var = casadi.Opti();

% Problem 1: Simple hanging chain
% Decision variables: positions
x = opti_var.variable(1, N);
y = opti_var.variable(1, N);

Vg = g*sum(m*y);
Vs = 0.5*D*sum((x(1:N-1)-x(2:N)).^2+(y(1:N-1)-y(2:N)).^2);

cost_NLP = Vg + Vs;
opti_var.minimize(cost_NLP);

opti_var.subject_to(x(1)   == -2);
opti_var.subject_to(y(1)   == 1);
opti_var.subject_to(x(end) == 2);
opti_var.subject_to(y(end) == 1);

opti_var.subject_to(y >= cos(0.1 * x) - 0.5);
opti_var.solver('ipopt');

sol_NLP = opti_var.solve();

% Plotting the results
figure; hold on;
plot(sol_NLP.value(x), sol_NLP.value(y), '-o');
xs = linspace(-2, 2, 1000);
plot(xs, cos(0.1 * xs) - 0.5, '--r');
grid on;

