clear all; close all; clc;
import casadi.*

%% QP problem
% Define constraints set
V = [-6.5,-0.25; -2, 3.45; 3.5, 3.45; 2.85,-4.5; -6.5,-0.25]; % vertices
X = Polyhedron('V', V); % requires installation of mpt3 toolbox

opti_var = casadi.Opti();  % Create an optimization solver
x = opti_var.variable(2, 1);  % Define a variable with dimensions 2x1

H = [0.2, 0.01; 0.09, 0.4]; 
H = H * H';  % Ensures H is positive definite
f = [0.1; 0.9];

% Define constraints and cost function
constraints = X.A * x <= X.b;  
cost_QP = x' * H * x / 2 + f' * x;

% Solver options
options.ipopt.print_level = 0;
options.ipopt.sb = 'yes';
options.print_time = 0;

% Set up the solver
opti_var.subject_to(constraints);
opti_var.minimize(cost_QP);
opti_var.solver('ipopt', options);

% Solve the problem and measure computation time
timer = tic;
solution = opti_var.solve();
time2 = toc(timer);

% Extract solution values
sol_val_qp = solution.value(opti_var.x);
fprintf('QP solution using CasADi = [%f %f]\n', sol_val_qp(1), sol_val_qp(2));

plot_QP(X, H, f, sol_val_qp);
